"""Functions to download trade data and code tables"""


import os
import ssl
import sys
import time
from urllib import error, request

from comexdown.tables import AUX_TABLES, TABLES

CANON_URL = "https://balanca.economia.gov.br/balanca/bd/"


def download_file(url, filepath=None, retry=3, blocksize=1024):
    """Downloads the file in `url` and saves it in `path`

    Parameters
    ----------
    url: str
        The resource's URL to download
    filepath: str
        The destination path of downloaded file
    retry: int [default=3]
        Number of retries until raising exception
    blocksize: int [default=1024]
        The block size of requests
    """

    if filepath:
        dirname, _ = os.path.split(filepath)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        dest = filepath
    else:
        dest = url.rsplit("/", maxsplit=1)[1]
    for x in range(retry):
        sys.stdout.write(f"Downloading: {url:<50} --> {dest}\n")
        sys.stdout.flush()
        try:
            resp = request.urlopen(url, context=ssl.SSLContext())
            length = resp.getheader("content-length")
            if length:
                length = int(length)

            size = 0
            with open(dest, "wb") as f:
                while True:
                    buf1 = resp.read(blocksize)
                    if not buf1:
                        break
                    f.write(buf1)
                    size += len(buf1)
                    p = size / length
                    bar = "[{:<70}]".format("=" * int(p * 70))
                    if size > 2**20:
                        size_txt = "{: >9.2f} MiB".format(size / 2**20)
                    else:
                        size_txt = "{: >9.2f} KiB".format(size / 2**10)
                    if length:
                        sys.stdout.write(
                            f"{bar} {p*100: >5.1f}% {size_txt}\r")
                        sys.stdout.flush()

        except error.URLError as e:
            sys.stdout.write(f"\nError... {e}")
            sys.stdout.flush()
            time.sleep(3)
            if x == retry - 1:
                raise

        else:
            sys.stdout.write("\n")
            sys.stdout.flush()
            break


def table(table_name, path):
    download_file(CANON_URL + "tabelas/" + AUX_TABLES[table_name], path)


def exp(year, path):
    """Downloads a exp file

    Parameters
    ----------
    year: int
        exp year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/ncm/EXP_{year}.csv".format(year=year)
    download_file(url, path)


def imp(year, path):
    """Downloads a imp file

    Parameters
    ----------
    year: int
        imp year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/ncm/IMP_{year}.csv".format(year=year)
    download_file(url, path)


def exp_mun(year, path):
    """Downloads a exp_mun file

    Parameters
    ----------
    year: int
        exp_mun year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/mun/EXP_{year}_MUN.csv".format(year=year)
    download_file(url, path)


def imp_mun(year, path):
    """Downloads a imp_mun file

    Parameters
    ----------
    year: int
        imp_mun year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/mun/IMP_{year}_MUN.csv".format(year=year)
    download_file(url, path)


def exp_nbm(year, path):
    """Downloads a exp_nbm file

    Parameters
    ----------
    year: int
        exp_nbm year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/nbm/EXP_{year}_NBM.csv".format(year=year)
    download_file(url, path)


def imp_nbm(year, path):
    """Downloads a imp_nbm file

    Parameters
    ----------
    year: int
        imp_nbm year to download
    path: str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/nbm/IMP_{year}_NBM.csv".format(year=year)
    download_file(url, path)


def exp_complete(path):
    """Downloads the file with complete data of exp

    Parameters
    ----------
    path : str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/ncm/EXP_COMPLETA.zip"
    download_file(url, path)


def imp_complete(path):
    """Downloads the file with complete data of imp

    Parameters
    ----------
    path : str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/ncm/IMP_COMPLETA.zip"
    download_file(url, path)


def exp_mun_complete(path):
    """Downloads the file with complete data of exp_mun

    Parameters
    ----------
    path : str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/mun/EXP_COMPLETA_MUN.zip"
    download_file(url, path)


def imp_mun_complete(path):
    """Downloads the file with complete data of imp_mun

    Parameters
    ----------
    path : str
        Destination path directory to save file

    """
    url = CANON_URL + "comexstat-bd/mun/IMP_COMPLETA_MUN.zip"
    download_file(url, path)


def agronegocio(path):
    """Downloads agronegocio file

    Parameters
    ----------
    path : str
        Destination path directory to save file

    """
    url = TABLES["agronegocio"]["url"]
    download_file(url, path)



"""Brazil's foreign trade data downloader"""


from comexdown import download, fs

__version__ = "1.3.3"


def get_year(path, year, exp=False, imp=False, mun=False):
    """Download trade data

    Parameters
    ----------
    path : str
        Destination path to save downloaded data, by default None
    year : int
        Year to download
    exp : bool, optional
        If True, download exports data, by default False
    imp : bool, optional
        If True, download imports data, by default False
    mun : bool, optional
        If True, download municipality data, by default False
    """
    if mun:
        if exp:
            download.exp_mun(
                year=year,
                path=fs.path_trade(
                    root=path,
                    direction="exp",
                    year=year,
                    mun=True,
                ),
            )
        if imp:
            download.imp_mun(
                year=year,
                path=fs.path_trade(
                    root=path,
                    direction="imp",
                    year=year,
                    mun=True,
                ),
            )
    else:
        if exp:
            download.exp(
                year=year,
                path=fs.path_trade(
                    root=path,
                    direction="exp",
                    year=year,
                    mun=False,
                ),
            )
        if imp:
            download.imp(
                year=year,
                path=fs.path_trade(
                    root=path,
                    direction="imp",
                    year=year,
                    mun=False,
                ),
            )


def get_year_nbm(path, year, exp=False, imp=False):
    """Download older trade data

    Parameters
    ----------
    path : str
        Destination path to save downloaded data, by default None
    year : int
        Year to download
    exp : bool, optional
        If True, download export data, by default False
    imp : bool, optional
        If True, download import data, by default False
    """
    if exp:
        download.exp_nbm(
            year=year,
            path=fs.path_trade_nbm(root=path, direction="exp", year=year),
        )
    if imp:
        download.imp_nbm(
            year=year,
            path=fs.path_trade_nbm(root=path, direction="imp", year=year),
        )


def get_complete(path, exp=False, imp=False, mun=False):
    """Download complete trade data

    Parameters
    ----------
    path : str
        Destination path to save downloaded data, by default "."
    exp : bool, optional
        If True, download complete export data, by default False
    imp : bool, optional
        If True, download complete import data, by default False
    mun : bool, optional
        If True, download complete municipality trade data, by default False
    """
    if mun:
        if exp:
            download.exp_mun_complete(path)
        if imp:
            download.imp_mun_complete(path)
    else:
        if exp:
            download.exp_complete(path)
        if imp:
            download.imp_complete(path)


def get_table(path, table):
    """Download auxiliary code tables

    Parameters
    ----------
    path : str
        Destination path to save downloaded code table
    table : str
        Name of auxiliary code table to download
    """
    if table == "agronegocio":
        download.agronegocio(
            path=fs.path_aux(root=path, name=table),
        )
        return
    download.table(
        table_name=table,
        path=fs.path_aux(root=path, name=table),
    )



"""Functions to manage files downloaded.

root
├───auxiliary_tables
├───exp
├───imp
├───mun_exp
├───mun_imp
├───nbm_exp
└───nbm_imp

"""


import pathlib
from typing import Union

from comexdown.tables import TABLES


def path_aux(
    root: Union[pathlib.PurePath, str],
    name: str,
) -> str:
    if isinstance(root, str):
        root = pathlib.Path(root)
    file_info = TABLES.get(name)
    if not file_info:
        return
    filename = file_info.get("file_ref")
    path = root / "auxiliary_tables" / filename
    return path


def path_trade(
    root: Union[pathlib.PurePath, str],
    direction: str,
    year: int,
    mun: bool = False,
) -> str:
    if isinstance(root, str):
        root = pathlib.Path(root)
    prefix = sufix = ""
    if direction.lower() == "exp":
        prefix = "EXP_"
    elif direction.lower() == "imp":
        prefix = "IMP_"
    else:
        raise ValueError(f"Invalid argument direction={direction}")
    if mun:
        sufix = "_MUN"
        direction = "mun_" + direction
    return root / direction / f"{prefix}{year}{sufix}.csv"


def path_trade_nbm(
    root: Union[pathlib.PurePath, str],
    direction: str,
    year: int,
) -> None:
    if isinstance(root, str):
        root = pathlib.Path(root)
    prefix = ""
    if direction.lower() == "exp":
        prefix = "EXP_"
    elif direction.lower() == "imp":
        prefix = "IMP_"
    else:
        raise ValueError(f"Invalid argument direction={direction}")
    direction = "nbm_" + direction
    return root / direction / f"{prefix}{year}_NBM.csv"
